USE [master] 
GO 

/****** Object:  StoredProcedure [dbo].[sp_Find2]    Script Date: 04/18/2012 13:38:55 ******/ 
SET ANSI_NULLS ON 
GO 

SET QUOTED_IDENTIFIER ON 
GO 

CREATE PROCEDURE [dbo].[sp_Find2] 
@SearchText1 varchar ( 100 ) = '' , 
@SearchText2 varchar ( 100 ) = '' , 
@SearchText3 varchar ( 100 ) = '' , 
@SearchText4 varchar ( 100 ) = '' , 
@DBName sysname = NULL 
-- , @PreviewTextSize int = 200 
, 
@SearchDBsFlag char ( 1 ) = 'Y' , 
@SearchJobsFlag char ( 1 ) = 'Y' , 
@SearchSSISFlag char ( 1 ) = 'Y' 
AS 
/*
* Created: 12/19/06, Michael F. Berry (SQL Server Magazine contributor)
*
* Modified: 01/25/07, Michael F. Berry, Make it output to one main recordset for clarity
* Modified: 09/04/08, Bill Lescher and Chase Jones, Updated for SQL2005 and added Jobs & SSIS Packages
* Modified: 07/22/09, Bill L, Returning the PreviewText
* Modified: 04/17/12, Said Khorramshahgol, Enabled searching for Tables & Columns
* Modified: 04/17/12, Said Khorramshahgol, Enabled searching for multiple keywords
* Modified: 04/17/12, Said Khorramshahgol, PreviewText column now shows column size
* Modified: 04/17/12, Said Khorramshahgol, Enabled search in Master and MSDB databases
* Modified: 04/17/12, Said Khorramshahgol, PreviewText column size is now set at 200 characters
*
* Description: Find any string within the T-SQL code on this SQL Server instance, specifically
*    Database objects and/or SQL Agent Jobs and/or SSIS Packages
*
* Test: sp_Find4 'KEYWORD 1' -- Search for 1 keyword across all objects/DBs/SSIS/Jobs
*  sp_Find4 'KEYWORD 1', 'KEYWORD2' -- Search for 2 keywords across all objects/DBs/SSIS/Jobs
*  sp_Find4 'track', NULL, NULL, NULL, 'Common' 
*  sp_Find4 'track', NULL, NULL, NULL, 'Common', 'Y', 'N', 'N' --DB Only
*  sp_Find4 'track', NULL, NULL, NULL, 'Common', 'N', 'N', 'Y' --SSIS Only
*/ 
SET TRANSACTION ISOLATION LEVEL READ Uncommitted; 
SET NOCOUNT On; 

DECLARE @PreviewTextSize int 
SET @PreviewTextSize = 200 

CREATE TABLE #FoundObject 
( 
    DatabaseName sysname 
    , ObjectName sysname 
    , ObjectTypeDesc nvarchar ( 60 ) , PreviewText varchar ( max ) ) 
--To show a little bit of the code and other info

DECLARE @SQL AS nvarchar ( max ) ; 

/**************************
*  Database Search
***************************/ 
IF @SearchDBsFlag = 'Y' 
BEGIN 
    IF @DBName IS NULL --Loop through all normal user databases
    BEGIN 
        DECLARE ObjCursor CURSOR Local Fast_Forward FOR 
        SELECT [Name] FROM Master.sys.Databases WHERE [Name] NOT IN ( 'AdventureWorks' , 'AdventureWorksDW' , 'Distribution' , 'Model' , 'TempDB' ) 
        -- ('AdventureWorks', 'AdventureWorksDW', 'Distribution', 'Master', 'MSDB', 'Model', 'TempDB');

        OPEN ObjCursor; 

        FETCH NEXT FROM ObjCursor INTO @DBName; 
        WHILE @@Fetch_Status = 0 
        BEGIN 
            SELECT @SQL = '
            Use [' + @DBName + ']

            Insert Into #FoundObject (
            DatabaseName
            , ObjectName
            , ObjectTypeDesc
            , PreviewText)
            Select Distinct
            ''' + @DBName + '''
            , sch.[Name] + ''.'' + obj.[Name] as ObjectName
            , obj.Type_Desc
            , Replace(Replace(SubString(mod.Definition, CharIndex(''' + @SearchText1 + ''', mod.Definition) - ' + cast ( @PreviewTextSize / 2 AS varchar ) + ', ' + 
            cast ( @PreviewTextSize AS varchar ) + '), char(13) + char(10), ''''), ''' + @SearchText1 + ''', ''***' + @SearchText1 + '***'')
            From  sys.objects obj 
            Inner Join sys.SQL_Modules mod On obj.Object_Id = mod.Object_Id
            Inner Join sys.Schemas sch On obj.Schema_Id = sch.Schema_Id
            Where mod.Definition Like ''%' + @SearchText1 + '%''
            AND  mod.Definition Like ''%' + coalesce ( @SearchText2 , '' ) + '%''
            AND  mod.Definition Like ''%' + coalesce ( @SearchText3 , '' ) + '%''
            AND  mod.Definition Like ''%' + coalesce ( @SearchText4 , '' ) + '%''' ; 
            --    Order By ObjectName';

            EXEC dbo.sp_executesql @SQL; 

            SELECT @SQL = '
            Use [' + @DBName + ']

            Insert Into #FoundObject (
            DatabaseName
            , ObjectName
            , ObjectTypeDesc
            , PreviewText)
            Select TABLE_CATALOG, TABLE_SCHEMA+''.''+TABLE_NAME, TABLE_TYPE, ''Table definitions are not set up yet''
            From  information_schema.tables
            Where TABLE_NAME like ''%' + @SearchText1 + '%''
            AND TABLE_NAME LIKE ''%' + coalesce ( @SearchText2 , '' ) + '%''
            AND TABLE_NAME LIKE ''%' + coalesce ( @SearchText3 , '' ) + '%''
            AND TABLE_NAME LIKE ''%' + coalesce ( @SearchText4 , '' ) + '%''' ; 

            EXEC dbo.sp_executesql @SQL; 

            SELECT @SQL = '
            Use [' + @DBName + ']

            Insert Into #FoundObject (
            DatabaseName
            , ObjectName
            , ObjectTypeDesc
            , PreviewText)
            Select TABLE_CATALOG, TABLE_SCHEMA+''.''+TABLE_NAME+''.''+COLUMN_NAME, ''COLUMN'', DATA_TYPE + ''('' + CAST(CHARACTER_MAXIMUM_LENGTH AS VARCHAR(100)) + '')''
            From  information_schema.columns
            Where COLUMN_NAME like ''%' + @SearchText1 + '%''
            AND COLUMN_NAME LIKE ''%' + coalesce ( @SearchText2 , '' ) + '%''
            AND COLUMN_NAME LIKE ''%' + coalesce ( @SearchText3 , '' ) + '%''
            AND COLUMN_NAME LIKE ''%' + coalesce ( @SearchText4 , '' ) + '%''' ; 

            EXEC dbo.sp_executesql @SQL; 

            FETCH NEXT FROM ObjCursor INTO @DBName; 
            End; 

            CLOSE ObjCursor; 

            DEALLOCATE ObjCursor; 
        END 
        ELSE --Only look through given database
        BEGIN 
            SELECT @SQL = '
            Use [' + @DBName + ']

            Insert Into #FoundObject (
            DatabaseName
            , ObjectName
            , ObjectTypeDesc
            , PreviewText)
            Select Distinct
            ''' + @DBName + '''
            , sch.[Name] + ''.'' + obj.[Name] as ObjectName
            , obj.Type_Desc
            , Replace(Replace(SubString(mod.Definition, CharIndex(''' + @SearchText1 + ''', mod.Definition) - ' + cast ( @PreviewTextSize / 2 AS varchar ) + ', ' + 
            cast ( @PreviewTextSize AS varchar ) + '), char(13) + char(10), ''''), ''' + @SearchText1 + ''', ''***' + @SearchText1 + '***'')
            From  sys.objects obj 
            Inner Join sys.SQL_Modules mod On obj.Object_Id = mod.Object_Id
            Inner Join sys.Schemas sch On obj.Schema_Id = sch.Schema_Id
            Where mod.Definition Like ''%' + @SearchText1 + '%'' 
            AND  mod.Definition Like ''%' + coalesce ( @SearchText2 , '' ) + '%''
            AND  mod.Definition Like ''%' + coalesce ( @SearchText3 , '' ) + '%''
            AND  mod.Definition Like ''%' + coalesce ( @SearchText4 , '' ) + '%''' ; 

            EXEC dbo.sp_ExecuteSQL @SQL; 
            End; 

            SELECT 'Database Objects' AS SearchType; 

            SELECT 
            DatabaseName , 
            ObjectName , 
            ObjectTypeDesc AS ObjectType , 
            PreviewText 
            FROM 
            #FoundObject 
            ORDER BY DatabaseName , 
            ObjectName; 

        END 

        /**************************
        *  Job Search
        ***************************/ 
        IF @SearchJobsFlag = 'Y' 
        BEGIN 
            SELECT 'Job Steps' AS SearchType; 

            SELECT 
            j. [Name] AS [Job Name] , 
            s.Step_Id AS [Step #] , 
            replace ( replace ( substring ( s.Command , charindex ( @SearchText1 , s.Command ) - @PreviewTextSize / 2 , @PreviewTextSize ) , char ( 13 ) + char ( 10 ) , '' ) , @SearchText1 , '***' + @SearchText1 + '***' ) AS Command 
            FROM MSDB.dbo.sysJobs j 
            INNER JOIN MSDB.dbo.sysJobSteps s ON j.Job_Id = s.Job_Id 
            WHERE s.Command LIKE '%' + @SearchText1 + '%' 
            AND s.Command LIKE '%' + coalesce ( @SearchText2 , '' ) + '%' 
            AND s.Command LIKE '%' + coalesce ( @SearchText3 , '' ) + '%' 
            AND s.Command LIKE '%' + coalesce ( @SearchText4 , '' ) + '%' ; 

        END 

        /**************************
        *  SSIS Search
        ***************************/ 
        IF @SearchSSISFlag = 'Y' 
        BEGIN 
            SELECT 'SSIS Packages' AS SearchType; 

            SELECT 
            [Name] AS [SSIS Name] , 
            replace 
            ( replace 
                ( substring 
                    ( cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) , charindex ( @SearchText1 , cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) ) - 
                        @PreviewTextSize / 2 , @PreviewTextSize ) , char ( 13 ) + char ( 10 ) , '' ) , @SearchText1 , '***' + @SearchText1 + '***' ) 
            AS [SSIS XML] 
            FROM MSDB.dbo.sysDTSPackages90 
            WHERE cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) LIKE '%' + @SearchText1 + '%' 
            AND cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) LIKE '%' + coalesce ( @SearchText2 , '' ) + '%' 
            AND cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) LIKE '%' + coalesce ( @SearchText3 , '' ) + '%' 
            AND cast ( cast ( PackageData AS varbinary ( max ) ) AS varchar ( max ) ) LIKE '%' + coalesce ( @SearchText4 , '' ) + '%' ; 

        END 

        GO